
if DATABASEPROPERTY('testdb', 'Version') is not null
drop database testdb
go

create database testdb
go

use testdb
go

-- 
-- Step 1:
-- Create tables for orders and order items
--

create table dbo.TVPOrd (
  CustCode varchar(5),
  OrdNo int identity,
  OrdDate datetime
)
go

create table dbo.TVPItem (
  OrdNo int,
  ProdCode int,
  Qty int
)
go

-- 
-- Step 2: Create procedures to insert a single order and a single item
--
create procedure dbo.TVPOrderInsert
(
	@CustCode varchar(5), 
	@OrdNo integer output, 
	@OrdDate datetime output)
as 
	set @OrdDate = GETDATE();
	insert into dbo.TVPOrd (OrdDate, CustCode) 
		values (@OrdDate, @CustCode); 
	select @OrdNo = SCOPE_IDENTITY()
go

-- old way: two procedures and multiple calls needed
create procedure dbo.TVPItemInsert
(
	@OrdNo integer, 
	@ProdCode integer, 
	@Qty integer)
as 
	insert into dbo.TVPItem (OrdNo, ProdCode, Qty) 
		values (@OrdNo, @ProdCode, @Qty)
go

-- old way: 4 database roundtrips
declare @ordno int;
declare @orddate datetime;
exec dbo.TVPOrderInsert 'ALFKI', @ordno output, @orddate output
-- insert three lines items
exec dbo.TVPItemInsert @ordno, 111, 1
exec dbo.TVPItemInsert @ordno, 222, 2
exec dbo.TVPItemInsert @ordno, 333, 3
go
select * from dbo.TVPOrd;
select * from dbo.TVPItem;
go
		
--
-- Step 3 Create a table-valued parameter 
--        This is actually creating a user-defined type, 
--        that is a strongly typed table
--		
				
create type dbo.TVPParam as table(ProdCode integer, Qty integer)
go

--
-- Step 4 Redo the order entry procedure 
--        to insert 1 Order and 1-n line items in a single round trip
--

create procedure dbo.TVPOrderEntry
(
	@CustCode varchar(5), 
	@Items TVPParam READONLY,  -- note that the table-valued parameter is read only
	@OrdNo integer output, 
	@OrdDate datetime output)
as 
	set @OrdDate = GETDATE();
	insert into TVPOrd (OrdDate, CustCode) 
		values (@OrdDate, @CustCode); 
	select @OrdNo = SCOPE_IDENTITY(); 
	insert into TVPItem (OrdNo, ProdCode, Qty) 
		select @OrdNo, ProdCode, Qty from @Items
go

-- single database roundtrip
declare @ordno int;
declare @orddate datetime;
declare @t TVPParam ;
insert into @t values(444, 4), (555,5), (666,6);
exec dbo.TVPOrderEntry 'BEAUC', @t, @ordno output, @orddate output;
go
select * from dbo.TVPOrd;
select * from dbo.TVPItem; 
go

--
-- Information about the table-value parameter type is stored in system views
--
select * from sys.types				
select * from sys.table_types
go

-- Table type is member of a schema
declare @t as dbo.TVPParam
go
