
-- Dependency tracking

if DATABASEPROPERTY('test', 'Version') is not null
DROP DATABASE test
GO

CREATE DATABASE test
GO

USE test
GO

-- New catalog view; replaces sys.sql_dependencies
-- Tracks both schema-bound and non-schema-bound dependencies
-- Tracks cross-database and cross-server references (by name)
select * from sys.sql_expression_dependencies

-- New dynamic management function; replaces sp_depends
-- Returns a row for each entity referenced by a given entity
-- E.g. show me all objects referenced in stored procedure p1
select * from sys.dm_sql_referenced_entities('dbo.t1', 'OBJECT')

-- New dynamic management function; replaces sp_depends
-- Returns a row for each entity that references a given entity
-- E.g. show me all objects that would be broken if I drop table t1
select * from sys.dm_sql_referencing_entities('dbo.t1', 'OBJECT')

CREATE TABLE dbo.t1 (id int, name varchar(20))
GO

CREATE TABLE dbo.t2 (id int, address varchar(20))
GO

CREATE PROCEDURE dbo.p2
AS
SELECT * FROM t1; 
GO

CREATE PROCEDURE p3
AS
UPDATE dbo.t1 SET name = 'Fred'
 WHERE id = 1;

CREATE VIEW v1 
WITH SCHEMABINDING
AS
SELECT t1.id, t1.name, t2.address 
FROM dbo.t1 
INNER JOIN dbo.t2 ON t1.id = t2.id
GO

CREATE SCHEMA s;
GO

CREATE FUNCTION s.foo (@x INT) 
RETURNS TABLE AS
RETURN
   SELECT * FROM t1 WHERE id < @x;
go

SELECT * FROM sys.dm_sql_referencing_entities ('dbo.t1', 'OBJECT');
go

SELECT * FROM sys.dm_sql_referenced_entities('dbo.p3', 'OBJECT')
go


