
-- this is to enable at an engine level
EXEC sp_filestream_configure 
    @enable_level = 3;
GO

-- change in CTP6
sp_configure 'show advanced options', 1
reconfigure
sp_configure 'filestream access level', '2'
reconfigure
go

USE master;
GO
IF EXISTS (
  SELECT * FROM sys.databases 
  WHERE name = N'FileStreamDB'
)
  DROP DATABASE FileStreamDB
GO
CREATE DATABASE FileStreamDB ON PRIMARY
  ( NAME = FileStreamDB_data, 
    FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10.MSSQLSERVER\MSSQL\Data\FileStreamDB_data.mdf',
    SIZE = 5MB,
    MAXSIZE = 50MB, 
    FILEGROWTH = 15%),
FILEGROUP FileStreamGroup1 CONTAINS FILESTREAM
  ( NAME = FileStreamDBResumes, 
    FILENAME = N'C:\Files\Resumes') -- the C:\files directory needs to exist before running 
LOG ON 
  ( NAME = 'FileStreamDB_log', 
    FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10.MSSQLSERVER\MSSQL\Data\FileStreamDB_log.ldf',
    SIZE = 1MB, 
    MAXSIZE = 25MB, 
    FILEGROWTH = 5MB);
GO

USE FileStreamDB;
GO
IF OBJECT_ID('dbo.student', 'U') IS NOT NULL
  DROP TABLE dbo.student
GO
CREATE TABLE dbo.student
(student_id UNIQUEIDENTIFIER ROWGUIDCOL NOT NULL UNIQUE, 
  Name varchar(25),
  Resume varbinary(max) FILESTREAM);
GO

-- no file
USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Alec', NULL);


USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Alec2', NULL);

USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Bob', CAST ('' as varbinary(max)));
    
USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Cecille', 
      CAST ('My resume text' as varbinary(max)));
      
-- SELECT data
SELECT * FROM dbo.student;
GO
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;

-- UPDATE data
USE FileStreamDB;
GO
UPDATE dbo.student
SET Resume = CAST ('My updated resume text' as varbinary(max))
WHERE Name = 'Cecille';
GO

-- DELETE data
USE FileStreamDB;
GO
DELETE dbo.student
WHERE Name = 'Bob';
GO

-- PathName() is case sensitive
SELECT student.Resume.PathName()
  FROM dbo.student;


CREATE TABLE t
    (
    Id int
    ,RowGuidColumn UNIQUEIDENTIFIER NOT NULL UNIQUE ROWGUIDCOL
    ,FileStreamColumn varbinary(max) FILESTREAM
    );
    
INSERT INTO t VALUES(1, newid(), CAST ('Some text' as varbinary(max)));
 

-- ROWGUID Columns in Views
-- SELECT FileStreamColumn.PathName() will succeed with View1, 
-- but fail with View2 because RowGuidColumn is not exposed.

-- Succeeds 

CREATE VIEW View1 
AS
SELECT RowGuidColumn
    ,FileStreamColumn
FROM t;

-- Fails because it is missing the RowGuidColumn
CREATE VIEW View2 
AS
SELECT FileStreamColumn
FROM t;
 

-- ROWGUID Columns in Derived Tables
-- In the following example the first statement succeeds. 
-- The second fails because RowGuidColumn is not exposed in the derived table.

-- Succeeds 

SELECT FileStreamColumn.PathName() 
FROM (
      SELECT FileStreamColumn, RowGuidColumn
      FROM t
      ) AS tab ;

-- Fails because it is missing the RowGuidColumn
SELECT FileStreamColumn.PathName()
FROM (
      SELECT FileStreamColumn 
      FROM t
      );
 

-- ROWGUID Columns in Correlated Subqueries
-- In the following example the first statement succeeds. 
-- The second fails because RowGuidColumn is not exposed in the subquery.

-- Succeeds 
SELECT FileStreamColumn, RowGuidColumn 
FROM t 
WHERE EXISTS (SELECT * 
              FROM t2 
              WHERE SavedPath = FileStreamColumn.PathName());

-- Fails because it is missing the RowGuidColumn
SELECT FileStreamColumn 
FROM t 
WHERE EXISTS(SELECT * 
             FROM t2 
             WHERE savedpath = FileStreamColumn.PathName()); 
 

-- ROWGUID Columns in Scalar Subqueries
-- In the following example the first statement succeeds. The second fails because a scalar subquery can return only one column.

-- Succeeds 

 Copy Code 
SELECT t2.id, t.FileStreamColumn.PathName()
FROM t 
    INNER JOIN t2 
        ON t.id = t2.id; 

-- Fails because it is missing the RowGuidColumn
SELECT id, 
    (SELECT FileStreamColumn 
    FROM t 
    WHERE t.id = t2.id
    ).PathName() 
FROM t2;
 
 /*
DECLARE @PathName nvarchar(max)
SET @PathName = (
    SELECT TOP 1 photo.PathName()
    FROM dbo.Customer
    WHERE LastName = 'CustomerName'
    );

/* Step 1) Begin a transaction. */
BEGIN TRANSACTION

/* Step 2) Select the path of a FILESTREAM NTFS file into a variable. */
DECLARE @PathName nvarchar(max);
SET @PathName = (
    SELECT Photo.PathName()
    FROM dbo.Customer
    WHERE CustomerID = 102
    );

/* Step 3) Get the transaction context. */
DECLARE @TransactionToken varbinary(max);
SET @TransactionToken = SELECT GET_FILESTREAM_TRANSACTION_CONTEXT ();

/* Step 4) Call the OpenSqlFilestream API to obtain a Win32 handle by passing in the logical path (returned by.PathName()) and @TransactionToken and use the handle for streaming I/O. */

/* Step 5)  Call ReadFile(), WriteFile(), TransitFile(), SetFilePointer(), SetEndofFile() or FlushFileBuffers(). */

/* Step 6) Call CloseHandle() to close the handle returned in Step 4. */

/* Step 7) Optionally, use OLE DB to issue a query on the connection used in Step 4 */

/* Step 8) Commit or roll back the transaction. */
COMMIT TRANSACTION
*/

-- Isolation levels

ALTER DATABASE FILESTREAMDB SET ALLOW_SNAPSHOT_ISOLATION ON
GO

ALTER DATABASE FILESTREAMDB SET READ_COMMITTED_SNAPSHOT ON
GO

SET TRANSACTION ISOLATION LEVEL SNAPSHOT
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;

SET TRANSACTION ISOLATION LEVEL REPEATABLE READ
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
begin tran
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;
commit

ALTER DATABASE FILESTREAMDB SET READ_COMMITTED_SNAPSHOT OFF
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED
begin tran
UPDATE dbo.student
SET Resume = CAST ('12345' as varbinary(max))
WHERE Name = 'Cecille';
commit

select * from dbo.student

delete dbo.student where name = 'Fred'






