
use AdventureWorks
Go

--
-- Common Hierarchical Queries on the table
--

-- 1. Find subordintes 
DECLARE @CurrentEmployee hierarchyid

SELECT @CurrentEmployee = OrgNode
FROM HumanResources.EmployeeOrg
WHERE EmployeeID = 46 ;

SELECT @CurrentEmployee


SELECT *
FROM HumanResources.EmployeeOrg
WHERE @CurrentEmployee.IsDescendant(OrgNode) = 1 ;

-- 2. Get ancestor (to return descendant)
DECLARE @CurrentEmployee hierarchyid

SELECT @CurrentEmployee = OrgNode
FROM HumanResources.EmployeeOrg
WHERE EmployeeID = 46 ;

SELECT OrgNode.ToString() AS Text_OrgNode, *
FROM HumanResources.EmployeeOrg
WHERE OrgNode.GetAncestor(1) = @CurrentEmployee

-- 3. Get Ancestor (to return descendents two levels deep)
DECLARE @CurrentEmployee hierarchyid

SELECT @CurrentEmployee = OrgNode
FROM HumanResources.EmployeeOrg
WHERE EmployeeID = 6 ;

SELECT OrgNode.ToString() AS Text_OrgNode, *
FROM HumanResources.EmployeeOrg
WHERE OrgNode.GetAncestor(2) = @CurrentEmployee

-- 4. Get all levels of hierarchy
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode.GetLevel() AS EmpLevel, *
FROM HumanResources.EmployeeOrg 
GO

-- 5. Get root
SELECT OrgNode.ToString() AS Text_OrgNode, *
FROM HumanResources.EmployeeOrg
WHERE OrgNode = hierarchyid::GetRoot() ;
GO

--
-- Hierarchical Maintanance Routines on the table
--

-- 1. Wanida moves from being Sariya's assistant to Jill's assistant
DECLARE @CurrentEmployee hierarchyid , @OldParent hierarchyid, @NewParent hierarchyid
SELECT @CurrentEmployee = OrgNode FROM HumanResources.EmployeeOrg -- Wanida
  WHERE EmployeeID = 269 ; 
SELECT @OldParent = OrgNode FROM HumanResources.EmployeeOrg  -- Sariya
  WHERE EmployeeID = 46 ; 
SELECT @NewParent = OrgNode FROM HumanResources.EmployeeOrg -- Jill
  WHERE EmployeeID = 119 ; 

UPDATE HumanResources.EmployeeOrg
SET OrgNode = @CurrentEmployee.Reparent(@OldParent, @NewParent) 
WHERE OrgNode = @CurrentEmployee ;
GO

-- check the population of the hierarchy
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO

-- add a new intern (expands hierarchy)
EXEC AddEmp 269, 291, 'Kevin', 'Marketing Intern'  ;
GO

-- check the population of the hierarchy
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO

-- 2. Move Jill's subordinates back to Sariya
DECLARE @OldParent hierarchyid, @NewParent hierarchyid
SELECT @OldParent = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 119 ; -- Jill
SELECT @NewParent = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 46 ; -- Sariya

UPDATE HumanResources.EmployeeOrg
SET OrgNode = OrgNode.Reparent(@OldParent, @NewParent) 
WHERE 
@OldParent.IsDescendant(OrgNode) = 1 
AND EmployeeID <> 119 -- This excludes Jill from the move.

-- check the population of the hierarchy
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO