
use tempdb
GO

-- Adapted from "Definitive XML Schema" by Priscilla Walmsley

-- schema that uses union and list productions
CREATE XML SCHEMA COLLECTION coll1 AS
'
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">

  <xs:element name="sizes" type="SizesType"/>

  <xs:complexType name="SizesType">
    <xs:choice maxOccurs="unbounded">
      <xs:element name="size" type="SizeType"/>
      <xs:element name="internationalSize"
                   type="InternationalSizeType"/>
      <xs:element name="availableSizes"
                   type="AvailableSizesType"/>
      <xs:element name="applicableSizes"
                   type="ApplicableSizesType"/>
    </xs:choice>
  </xs:complexType>

  <!-- union support -->
  <xs:simpleType name="SizeType">
    <xs:union memberTypes="DressSizeType">
      <xs:simpleType>
        <xs:restriction base="xs:token">
          <xs:enumeration value="small"/>
          <xs:enumeration value="medium"/>
          <xs:enumeration value="large"/>
        </xs:restriction>
      </xs:simpleType>
    </xs:union>
  </xs:simpleType>

  <xs:simpleType name="DressSizeType">
    <xs:restriction base="xs:integer">
      <xs:minInclusive value="2"/>
      <xs:maxInclusive value="18"/>
    </xs:restriction>
  </xs:simpleType>

  <xs:simpleType name="InternationalSizeType">
    <xs:union memberTypes="SizeType">
      <xs:simpleType>
        <xs:restriction base="xs:integer">
          <xs:minInclusive value="24"/>
          <xs:maxInclusive value="54"/>
        </xs:restriction>
      </xs:simpleType>
    </xs:union>
  </xs:simpleType>

  <xs:simpleType name="AvailableSizesType">
    <xs:list itemType="SizeType"/>
  </xs:simpleType>

  <!-- list support -->
  <xs:simpleType name="ApplicableSizesType">
    <xs:restriction>
      <xs:simpleType>
        <xs:list itemType="SizeType"/>
      </xs:simpleType>
      <xs:enumeration value="small medium large"/>
      <xs:enumeration value="2 4 6 8 10 12 14 16 18"/>
    </xs:restriction>
  </xs:simpleType>

</xs:schema>
'
GO

-- instance
DECLARE @x XML (dbo.coll1)
SET @X = 
'
<sizes xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
       xsi:noNamespaceSchemaLocation="chapter11.xsd">
  <size>12</size>
  <size>medium</size>
  <!-- <smallSize>6</smallSize>
  <smallSize>small</smallSize> -->
  <internationalSize>12</internationalSize>
  <internationalSize>24</internationalSize>
  <internationalSize>large</internationalSize>
  <availableSizes>10 large 2</availableSizes>
  <applicableSizes>small medium large</applicableSizes>
</sizes>
'
-- validation error: no smallSize
SET @X = 
'
<sizes xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
       xsi:noNamespaceSchemaLocation="chapter11.xsd">
  <size>12</size>
  <size>medium</size>
  <smallSize>6</smallSize>
  <smallSize>small</smallSize>
  <internationalSize>12</internationalSize>
  <internationalSize>24</internationalSize>
  <internationalSize>large</internationalSize>
  <availableSizes>10 large 2</availableSizes>
  <applicableSizes>small medium large</applicableSizes>
</sizes>
'