
if DATABASEPROPERTY('testdb', 'Version') is not null
drop database testdb
go

create database testdb
go

use testdb
go

CREATE QUEUE receiveq
go
-- default means receiver can receive 
-- message type DEFAULT
CREATE SERVICE receiver
	ON QUEUE receiveq
	([DEFAULT])
go

-- RECEIVER CREATED

CREATE QUEUE senderq 
go

-- no contract because sender names the contract
-- in begin dialog
CREATE SERVICE sender
	ON QUEUE senderq
go

-- SENDER CREATED

DECLARE @h UNIQUEIDENTIFIER;
BEGIN TRAN;
BEGIN TRY
	BEGIN DIALOG CONVERSATION @h
	FROM SERVICE sender
	TO SERVICE 'receiver'
	WITH ENCRYPTION = OFF;

	SEND ON CONVERSATION @h
	('Hello World');
    COMMIT;
END TRY
BEGIN CATCH
	PRINT ERROR_MESSAGE()
    ROLLBACK;
END CATCH
-- don't end conversation here
-- END CONVERSATION @h;
go

-- MESSAGE SENT

DECLARE @h UNIQUEIDENTIFIER;
DECLARE @m VARCHAR(MAX);
BEGIN TRAN;
RECEIVE TOP(1) @h=conversation_handle, 
		@m=CAST(message_body as VARCHAR(MAX)) FROM receiveq;
PRINT @m;
-- dont ever end conversation with cleanup in a program
-- END CONVERSATION @h WITH CLEANUP;
END CONVERSATION @h
COMMIT;
go

-- MESSAGE RECEIVED

-- receive end of conversation at the sender
-- and end the conversation there
DECLARE @h UNIQUEIDENTIFIER;
DECLARE @m VARCHAR(MAX);
BEGIN TRAN;
RECEIVE TOP(1) @h=conversation_handle, 
		@m=CAST(message_body as VARCHAR(MAX)) FROM senderq;
PRINT @m;
END CONVERSATION @h
COMMIT;
go

-- better, have an activation program on senderq
-- that processes end conversation and errors

CREATE PROCEDURE inititator_queue_activated_procedure
AS
DECLARE @dh UNIQUEIDENTIFIER;
DECLARE @message_type SYSNAME;
DECLARE @message_body NVARCHAR(4000);
BEGIN TRANSACTION;
WAITFOR (
      RECEIVE @dh = [conversation_handle],
            @message_type = [message_type_name],
            @message_body = CAST([message_body] AS NVARCHAR(4000)) 
      FROM senderq), TIMEOUT 1000;
WHILE @dh IS NOT NULL
BEGIN
      IF @message_type = N'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
      BEGIN
            RAISERROR (N'Received error %s from service [receiver]', 10, 1, @message_body) WITH LOG;
      END
      END CONVERSATION @dh;
      COMMIT;
      SELECT @dh = NULL;
      BEGIN TRANSACTION;
      WAITFOR (
            RECEIVE @dh = [conversation_handle],
                  @message_type = [message_type_name],
                  @message_body = CAST([message_body] AS NVARCHAR(4000)) 
            FROM senderq), TIMEOUT 1000;
END
COMMIT;
GO
 
ALTER QUEUE senderq
      WITH ACTIVATION (
            STATUS = ON,
            MAX_QUEUE_READERS = 1,
            PROCEDURE_NAME = [inititator_queue_activated_procedure],
            EXECUTE AS OWNER);
GO






